package search;

import java.util.List;

/** A problem defined for use with an AI search algorithm. This class
 * must be extended with a specific class to define the problem, which
 * implements all of the abstract methods.
 * @author Jeff Heflin, Lehigh University (heflin@cse.lehigh.edu)
 *
 */
public abstract class SearchProblem {
  
  /** Return the initial state of the problem. */
  public abstract State getInitialState();
  
  /** Get the successors of the input state. To simplify costing, instead of only
   * returning <action,successor> pairs, we return step costs as well. This
   * eliminates the need for a separate step cost function that takes a node,
   * action and child state as input. The action is simply a string, and these
   * strings are used for displaying the steps of the solution. */
  public abstract List<Successor> getSuccessors(State s);
  
  /** Returns true if the parameter is a goal state. */
  public abstract boolean goalTest(State currentState);
  
  /** Returns the h(n) value for state parameter. Lower h(n) values are
   * estimated to be closer to the goal.
   */
  public abstract int getHeuristicValue(State state);
    
}
